(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = global || self, factory(global.ReactQuery = {}, global.React));
}(this, (function (exports, React) { 'use strict';

  React = React && Object.prototype.hasOwnProperty.call(React, 'default') ? React['default'] : React;

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  // TYPES
  var CancelledError = function CancelledError(options) {
    this.revert = options == null ? void 0 : options.revert;
    this.silent = options == null ? void 0 : options.silent;
  }; // UTILS

  var isServer = typeof window === 'undefined';
  function noop() {
    return undefined;
  }
  function functionalUpdate(updater, input) {
    return typeof updater === 'function' ? updater(input) : updater;
  }
  function defaultRetryDelay(attempt) {
    return Math.min(1000 * Math.pow(2, attempt), 30000);
  }
  function isValidTimeout(value) {
    return typeof value === 'number' && value >= 0 && value !== Infinity;
  }
  function isDocumentVisible() {
    // document global can be unavailable in react native
    if (typeof document === 'undefined') {
      return true;
    }

    return [undefined, 'visible', 'prerender'].includes(document.visibilityState);
  }
  function isOnline() {
    return navigator.onLine === undefined || navigator.onLine;
  }
  function ensureArray(value) {
    return Array.isArray(value) ? value : [value];
  }
  function uniq(array) {
    return array.filter(function (value, i) {
      return array.indexOf(value) === i;
    });
  }
  function difference(array1, array2) {
    return array1.filter(function (x) {
      return array2.indexOf(x) === -1;
    });
  }
  function replaceAt(array, index, value) {
    var copy = array.slice(0);
    copy[index] = value;
    return copy;
  }
  function timeUntilStale(updatedAt, staleTime) {
    return Math.max(updatedAt + (staleTime || 0) - Date.now(), 0);
  }
  function parseQueryArgs(arg1, arg2, arg3) {
    if (!isQueryKey(arg1)) {
      return arg1;
    }

    if (typeof arg2 === 'function') {
      return _extends({}, arg3, {
        queryKey: arg1,
        queryFn: arg2
      });
    }

    return _extends({}, arg2, {
      queryKey: arg1
    });
  }
  function parseFilterArgs(arg1, arg2, arg3) {
    return isQueryKey(arg1) ? [_extends({}, arg2, {
      queryKey: arg1
    }), arg3] : [arg1 || {}, arg2];
  }
  function matchQuery(filters, query) {
    var active = filters.active,
        exact = filters.exact,
        fetching = filters.fetching,
        fresh = filters.fresh,
        inactive = filters.inactive,
        predicate = filters.predicate,
        queryKey = filters.queryKey,
        stale = filters.stale;

    if (isQueryKey(queryKey)) {
      if (exact) {
        var hashFn = getQueryKeyHashFn(query.options);

        if (query.queryHash !== hashFn(queryKey)) {
          return false;
        }
      } else if (!partialDeepEqual(ensureArray(query.queryKey), ensureArray(queryKey))) {
        return false;
      }
    }

    var isActive;

    if (inactive === false || active && !inactive) {
      isActive = true;
    } else if (active === false || inactive && !active) {
      isActive = false;
    }

    if (typeof isActive === 'boolean' && query.isActive() !== isActive) {
      return false;
    }

    var isStale;

    if (fresh === false || stale && !fresh) {
      isStale = true;
    } else if (stale === false || fresh && !stale) {
      isStale = false;
    }

    if (typeof isStale === 'boolean' && query.isStale() !== isStale) {
      return false;
    }

    if (typeof fetching === 'boolean' && query.isFetching() !== fetching) {
      return false;
    }

    if (predicate && !predicate(query)) {
      return false;
    }

    return true;
  }
  function getQueryKeyHashFn(options) {
    return (options == null ? void 0 : options.queryKeyHashFn) || hashQueryKey;
  }
  /**
   * Default query keys hash function.
   */

  function hashQueryKey(queryKey) {
    return stableValueHash(queryKey);
  }
  /**
   * Hashes the value into a stable hash.
   */

  function stableValueHash(value) {
    return JSON.stringify(value, function (_, val) {
      return isPlainObject(val) ? Object.keys(val).sort().reduce(function (result, key) {
        result[key] = val[key];
        return result;
      }, {}) : val;
    });
  }
  /**
   * Checks if `b` partially matches with `a`.
   */

  function partialDeepEqual(a, b) {
    if (a === b) {
      return true;
    }

    if (typeof a !== typeof b) {
      return false;
    }

    if (typeof a === 'object') {
      return !Object.keys(b).some(function (key) {
        return !partialDeepEqual(a[key], b[key]);
      });
    }

    return false;
  }
  /**
   * This function returns `a` if `b` is deeply equal.
   * If not, it will replace any deeply equal children of `b` with those of `a`.
   * This can be used for structural sharing between JSON values for example.
   */

  function replaceEqualDeep(a, b) {
    if (a === b) {
      return a;
    }

    var array = Array.isArray(a) && Array.isArray(b);

    if (array || isPlainObject(a) && isPlainObject(b)) {
      var aSize = array ? a.length : Object.keys(a).length;
      var bItems = array ? b : Object.keys(b);
      var bSize = bItems.length;
      var copy = array ? [] : {};
      var equalItems = 0;

      for (var i = 0; i < bSize; i++) {
        var key = array ? i : bItems[i];
        copy[key] = replaceEqualDeep(a[key], b[key]);

        if (copy[key] === a[key]) {
          equalItems++;
        }
      }

      return aSize === bSize && equalItems === aSize ? a : copy;
    }

    return b;
  }
  /**
   * Shallow compare objects. Only works with objects that always have the same properties.
   */

  function shallowEqualObjects(a, b) {
    for (var key in a) {
      if (a[key] !== b[key]) {
        return false;
      }
    }

    return true;
  } // Copied from: https://github.com/jonschlinkert/is-plain-object

  function isPlainObject(o) {
    if (!hasObjectPrototype(o)) {
      return false;
    } // If has modified constructor


    var ctor = o.constructor;

    if (typeof ctor === 'undefined') {
      return true;
    } // If has modified prototype


    var prot = ctor.prototype;

    if (!hasObjectPrototype(prot)) {
      return false;
    } // If constructor does not have an Object-specific method


    if (!prot.hasOwnProperty('isPrototypeOf')) {
      return false;
    } // Most likely a plain Object


    return true;
  }

  function hasObjectPrototype(o) {
    return Object.prototype.toString.call(o) === '[object Object]';
  }

  function isQueryKey(value) {
    return typeof value === 'string' || Array.isArray(value);
  }
  function isCancelable(value) {
    return typeof (value == null ? void 0 : value.cancel) === 'function';
  }
  function isError(value) {
    return value instanceof Error;
  }
  function isCancelledError(value) {
    return value instanceof CancelledError;
  }
  function sleep(timeout) {
    return new Promise(function (resolve) {
      setTimeout(resolve, timeout);
    });
  }
  function getStatusProps(status) {
    return {
      status: status,
      isLoading: status === 'loading',
      isSuccess: status === 'success',
      isError: status === 'error',
      isIdle: status === 'idle'
    };
  }
  /**
   * Schedules a microtask.
   * This can be useful to schedule state updates after rendering.
   */

  function scheduleMicrotask(callback) {
    Promise.resolve().then(callback).catch(function (error) {
      return setTimeout(function () {
        throw error;
      });
    });
  }

  // GETTERS AND SETTERS
  // Default to a dummy "notify" implementation that just runs the callback
  var notifyFn = function notifyFn(callback) {
    callback();
  }; // Default to a dummy "batch notify" implementation that just runs the callback


  var batchNotifyFn = function batchNotifyFn(callback) {
    callback();
  };
  /**
   * Use this function to set a custom notify function.
   * This can be used to for example wrap notifications with `React.act` while running tests.
   */


  function setNotifyFn(fn) {
    notifyFn = fn;
  }
  /**
   * Use this function to set a custom function to batch notifications together into a single tick.
   * By default React Query will use the batch function provided by ReactDOM or React Native.
   */

  function setBatchNotifyFn(fn) {
    batchNotifyFn = fn;
  } // CLASS

  var NotifyManager = /*#__PURE__*/function () {
    function NotifyManager() {
      this.queue = [];
      this.transactions = 0;
    }

    var _proto = NotifyManager.prototype;

    _proto.batch = function batch(callback) {
      this.transactions++;
      var result = callback();
      this.transactions--;

      if (!this.transactions) {
        this.flush();
      }

      return result;
    };

    _proto.schedule = function schedule(callback) {
      if (this.transactions) {
        this.queue.push(callback);
      } else {
        scheduleMicrotask(function () {
          notifyFn(callback);
        });
      }
    };

    _proto.flush = function flush() {
      var queue = this.queue;
      this.queue = [];

      if (queue.length) {
        scheduleMicrotask(function () {
          batchNotifyFn(function () {
            queue.forEach(function (callback) {
              notifyFn(callback);
            });
          });
        });
      }
    };

    return NotifyManager;
  }(); // SINGLETON


  var notifyManager = new NotifyManager();

  // FUNCTIONS
  var logger = console || {
    error: noop,
    warn: noop,
    log: noop
  };
  function getLogger() {
    return logger;
  }
  function setLogger(c) {
    logger = c;
  }

  // CLASS
  var Query = /*#__PURE__*/function () {
    function Query(config) {
      this.setOptions(config.options);
      this.observers = [];
      this.cache = config.cache;
      this.queryKey = config.queryKey;
      this.queryHash = config.queryHash;
      this.state = getDefaultState(this.options);
      this.scheduleGc();
    }

    var _proto = Query.prototype;

    _proto.setOptions = function setOptions(options) {
      var _this$options$cacheTi;

      this.options = _extends({}, this.defaultOptions, options); // Default to 5 minutes if not cache time is set

      this.cacheTime = Math.max(this.cacheTime || 0, (_this$options$cacheTi = this.options.cacheTime) != null ? _this$options$cacheTi : 5 * 60 * 1000);
    };

    _proto.setDefaultOptions = function setDefaultOptions(options) {
      this.defaultOptions = options;
    };

    _proto.dispatch = function dispatch(action) {
      var _this = this;

      this.state = queryReducer(this.state, action);
      notifyManager.batch(function () {
        _this.observers.forEach(function (observer) {
          observer.onQueryUpdate(action);
        });

        _this.cache.notify(_this);
      });
    };

    _proto.scheduleGc = function scheduleGc() {
      var _this2 = this;

      this.clearGcTimeout();

      if (isServer || this.observers.length > 0 || !isValidTimeout(this.cacheTime)) {
        return;
      }

      this.gcTimeout = setTimeout(function () {
        _this2.remove();
      }, this.cacheTime);
    };

    _proto.cancel = function cancel(options) {
      var _this$cancelFetch;

      var promise = this.promise || Promise.resolve();
      (_this$cancelFetch = this.cancelFetch) == null ? void 0 : _this$cancelFetch.call(this, options);
      return promise.then(noop).catch(noop);
    };

    _proto.clearTimersObservers = function clearTimersObservers() {
      this.observers.forEach(function (observer) {
        observer.clearTimers();
      });
    };

    _proto.clearGcTimeout = function clearGcTimeout() {
      clearTimeout(this.gcTimeout);
      this.gcTimeout = undefined;
    };

    _proto.setData = function setData(updater, options) {
      var _this$options$isDataE, _this$options;

      var prevData = this.state.data; // Get the new data

      var data = functionalUpdate(updater, prevData); // Structurally share data between prev and new data if needed

      if (this.options.structuralSharing !== false) {
        data = replaceEqualDeep(prevData, data);
      } // Use prev data if an isDataEqual function is defined and returns `true`


      if ((_this$options$isDataE = (_this$options = this.options).isDataEqual) == null ? void 0 : _this$options$isDataE.call(_this$options, prevData, data)) {
        data = prevData;
      } // Set data and mark it as cached


      this.dispatch({
        data: data,
        hasNextPage: hasNextPage(this.options, data),
        hasPreviousPage: hasPreviousPage(this.options, data),
        pageParams: options == null ? void 0 : options.pageParams,
        type: 'success',
        updatedAt: options == null ? void 0 : options.updatedAt
      });
      return data;
    };

    _proto.remove = function remove() {
      this.cache.remove(this);
    };

    _proto.destroy = function destroy() {
      this.clearGcTimeout();
      this.clearTimersObservers();
      this.cancel();
    };

    _proto.isActive = function isActive() {
      return this.observers.some(function (observer) {
        return observer.options.enabled !== false;
      });
    };

    _proto.isFetching = function isFetching() {
      return this.state.isFetching;
    };

    _proto.isStale = function isStale() {
      return this.state.isInvalidated || !this.state.updatedAt || this.observers.some(function (observer) {
        return observer.getCurrentResult().isStale;
      });
    };

    _proto.isStaleByTime = function isStaleByTime(staleTime) {
      if (staleTime === void 0) {
        staleTime = 0;
      }

      return this.state.isInvalidated || !this.state.updatedAt || !timeUntilStale(this.state.updatedAt, staleTime);
    };

    _proto.onFocus = function onFocus() {
      this.onExternalEvent('focus');
    };

    _proto.onOnline = function onOnline() {
      this.onExternalEvent('online');
    };

    _proto.onExternalEvent = function onExternalEvent(type) {
      var _this$continueFetch;

      // Execute the first observer that wants to fetch on this event
      var fetchObserver = this.observers.find(function (observer) {
        var _observer$options = observer.options,
            enabled = _observer$options.enabled,
            refetchOnWindowFocus = _observer$options.refetchOnWindowFocus,
            refetchOnReconnect = _observer$options.refetchOnReconnect;

        var _observer$getCurrentR = observer.getCurrentResult(),
            isStale = _observer$getCurrentR.isStale;

        return enabled !== false && (type === 'focus' && (refetchOnWindowFocus === 'always' || refetchOnWindowFocus !== false && isStale) || type === 'online' && (refetchOnReconnect === 'always' || refetchOnReconnect !== false && isStale));
      });

      if (fetchObserver) {
        fetchObserver.refetch();
      } // Continue any paused fetch


      (_this$continueFetch = this.continueFetch) == null ? void 0 : _this$continueFetch.call(this);
    };

    _proto.subscribeObserver = function subscribeObserver(observer) {
      if (this.observers.indexOf(observer) !== -1) {
        return;
      }

      this.observers.push(observer); // Stop the query from being garbage collected

      this.clearGcTimeout();
      this.cache.notify(this);
    };

    _proto.unsubscribeObserver = function unsubscribeObserver(observer) {
      this.observers = this.observers.filter(function (x) {
        return x !== observer;
      });

      if (!this.observers.length) {
        // If the transport layer does not support cancellation
        // we'll let the query continue so the result can be cached
        if (this.isTransportCancelable) {
          this.cancel();
        }

        this.scheduleGc();
      }

      this.cache.notify(this);
    };

    _proto.invalidate = function invalidate() {
      if (!this.state.isInvalidated) {
        this.dispatch({
          type: 'invalidate'
        });
      }
    };

    _proto.fetch = function fetch(options, fetchOptions) {
      var _this3 = this;

      if (this.state.isFetching) if ((fetchOptions == null ? void 0 : fetchOptions.fetchMore) && this.state.updatedAt) {
        // Silently cancel current fetch if the user wants to fetch more
        this.cancel({
          silent: true
        });
      } else if (this.promise) {
        // Return current promise if we are already fetching
        return this.promise;
      } // Update config if passed, otherwise the config from the last execution is used

      if (options) {
        this.setOptions(options);
      } // Use the options from the first observer with a query function if no function is found.
      // This can happen when the query is hydrated or created with setQueryData.


      if (!this.options.queryFn) {
        var observer = this.observers.find(function (x) {
          return x.options.queryFn;
        });

        if (observer) {
          this.setOptions(observer.options);
        }
      } // Get the query function params


      var params = ensureArray(this.queryKey);
      var filter = this.options.queryFnParamsFilter;
      params = filter ? filter(params) : params;
      var promise = this.options.infinite ? this.startInfiniteFetch(this.options, params, fetchOptions) : this.startFetch(this.options, params);
      this.promise = promise.catch(function (error) {
        // Set error state if needed
        if (!(isCancelledError(error) && error.silent)) {
          _this3.dispatch({
            type: 'error',
            error: error
          });
        } // Log error


        if (!isCancelledError(error)) {
          getLogger().error(error);
        } // Propagate error


        throw error;
      });
      return this.promise;
    };

    _proto.startFetch = function startFetch(options, params) {
      var _this4 = this;

      // Create function to fetch the data
      var queryFn = options.queryFn || defaultQueryFn;

      var fetchData = function fetchData() {
        return queryFn.apply(void 0, params);
      }; // Set to fetching state if not already in it


      if (!this.state.isFetching) {
        this.dispatch({
          type: 'fetch'
        });
      } // Try to fetch the data


      return this.tryFetchData(options, fetchData).then(function (data) {
        return _this4.setData(data);
      });
    };

    _proto.startInfiniteFetch = function startInfiniteFetch(options, params, fetchOptions) {
      var _this5 = this;

      var queryFn = options.queryFn || defaultQueryFn;
      var fetchMore = fetchOptions == null ? void 0 : fetchOptions.fetchMore;
      var pageParam = fetchMore == null ? void 0 : fetchMore.pageParam;
      var isFetchingNextPage = (fetchMore == null ? void 0 : fetchMore.direction) === 'forward';
      var isFetchingPreviousPage = (fetchMore == null ? void 0 : fetchMore.direction) === 'backward';
      var oldPages = this.state.data || [];
      var oldPageParams = this.state.pageParams || [];
      var newPageParams = oldPageParams; // Create function to fetch a page

      var fetchPage = function fetchPage(pages, manual, param, previous) {
        if (typeof param === 'undefined' && !manual && pages.length) {
          return Promise.resolve(pages);
        }

        return Promise.resolve().then(function () {
          return queryFn.apply(void 0, params.concat([param]));
        }).then(function (page) {
          newPageParams = previous ? [param].concat(newPageParams) : [].concat(newPageParams, [param]);
          return previous ? [page].concat(pages) : [].concat(pages, [page]);
        });
      }; // Create function to fetch the data


      var fetchData = function fetchData() {
        // Reset new page params
        newPageParams = oldPageParams; // Fetch first page?

        if (!oldPages.length) {
          return fetchPage([]);
        } // Fetch next page?


        if (isFetchingNextPage) {
          var _manual = typeof pageParam !== 'undefined';

          var param = _manual ? pageParam : getNextPageParam(options, oldPages);
          return fetchPage(oldPages, _manual, param);
        } // Fetch previous page?


        if (isFetchingPreviousPage) {
          var _manual2 = typeof pageParam !== 'undefined';

          var _param = _manual2 ? pageParam : getPreviousPageParam(options, oldPages);

          return fetchPage(oldPages, _manual2, _param, true);
        } // Refetch pages


        newPageParams = [];
        var manual = typeof options.getNextPageParam === 'undefined'; // Fetch first page

        var promise = fetchPage([], manual, oldPageParams[0]); // Fetch remaining pages

        var _loop = function _loop(i) {
          promise = promise.then(function (pages) {
            var param = manual ? oldPageParams[i] : getNextPageParam(options, pages);
            return fetchPage(pages, manual, param);
          });
        };

        for (var i = 1; i < oldPages.length; i++) {
          _loop(i);
        }

        return promise;
      }; // Set to fetching state if not already in it


      if (!this.state.isFetching || this.state.isFetchingNextPage !== isFetchingNextPage || this.state.isFetchingPreviousPage !== isFetchingPreviousPage) {
        this.dispatch({
          type: 'fetch',
          isFetchingNextPage: isFetchingNextPage,
          isFetchingPreviousPage: isFetchingPreviousPage
        });
      } // Try to get the data


      return this.tryFetchData(options, fetchData).then(function (data) {
        return _this5.setData(data, {
          pageParams: newPageParams
        });
      });
    };

    _proto.tryFetchData = function tryFetchData(options, fn) {
      var _this6 = this;

      return new Promise(function (outerResolve, outerReject) {
        var resolved = false;
        var continueLoop;
        var cancelTransport;

        var done = function done() {
          resolved = true;
          delete _this6.cancelFetch;
          delete _this6.continueFetch;
          delete _this6.isTransportCancelable; // End loop if currently paused

          continueLoop == null ? void 0 : continueLoop();
        };

        var resolve = function resolve(value) {
          done();
          outerResolve(value);
        };

        var reject = function reject(value) {
          done();
          outerReject(value);
        }; // Create callback to cancel this fetch


        _this6.cancelFetch = function (cancelOptions) {
          reject(new CancelledError(cancelOptions));
          cancelTransport == null ? void 0 : cancelTransport();
        }; // Create callback to continue this fetch


        _this6.continueFetch = function () {
          continueLoop == null ? void 0 : continueLoop();
        }; // Create loop function


        var run = function run() {
          // Do nothing if already resolved
          if (resolved) {
            return;
          }

          var promiseOrValue; // Execute query

          try {
            promiseOrValue = fn();
          } catch (error) {
            promiseOrValue = Promise.reject(error);
          } // Check if the transport layer support cancellation


          if (isCancelable(promiseOrValue)) {
            cancelTransport = function cancelTransport() {
              try {
                promiseOrValue.cancel();
              } catch (_unused) {}
            };

            _this6.isTransportCancelable = true;
          }

          Promise.resolve(promiseOrValue).then(function (data) {
            // Resolve with data
            resolve(data);
          }).catch(function (error) {
            var _options$retry, _options$retryDelay;

            // Stop if the fetch is already resolved
            if (resolved) {
              return;
            } // Do we need to retry the request?


            var failureCount = _this6.state.failureCount;
            var retry = (_options$retry = options.retry) != null ? _options$retry : 3;
            var retryDelay = (_options$retryDelay = options.retryDelay) != null ? _options$retryDelay : defaultRetryDelay;
            var shouldRetry = retry === true || typeof retry === 'number' && failureCount < retry || typeof retry === 'function' && retry(failureCount, error);

            if (!shouldRetry) {
              // We are done if the query does not need to be retried
              reject(error);
              return;
            } // Increase the failureCount


            _this6.dispatch({
              type: 'failed'
            }); // Delay


            sleep(functionalUpdate(retryDelay, failureCount) || 0) // Pause if needed
            .then(function () {
              // Pause retry if the document is not visible or when the device is offline
              if (!isDocumentVisible() || !isOnline()) {
                return new Promise(function (continueResolve) {
                  continueLoop = continueResolve;
                });
              }
            }) // Try again
            .then(run);
          });
        }; // Start loop


        run();
      });
    };

    return Query;
  }();

  function defaultQueryFn() {
    return Promise.reject();
  }

  function getNextPageParam(options, pages) {
    return options.getNextPageParam == null ? void 0 : options.getNextPageParam(pages[pages.length - 1], pages);
  }

  function getPreviousPageParam(options, pages) {
    return options.getPreviousPageParam == null ? void 0 : options.getPreviousPageParam(pages[0], pages);
  }
  /**
   * Checks if there is a next page.
   * Returns `undefined` if it cannot be determined.
   */


  function hasNextPage(options, pages) {
    return options.getNextPageParam && Array.isArray(pages) ? typeof getNextPageParam(options, pages) !== 'undefined' : undefined;
  }
  /**
   * Checks if there is a previous page.
   * Returns `undefined` if it cannot be determined.
   */


  function hasPreviousPage(options, pages) {
    return options.getPreviousPageParam && Array.isArray(pages) ? typeof getPreviousPageParam(options, pages) !== 'undefined' : undefined;
  }

  function getDefaultState(options) {
    var data = typeof options.initialData === 'function' ? options.initialData() : options.initialData;
    var hasData = typeof data !== 'undefined';
    return {
      data: data,
      dataUpdateCount: 0,
      error: null,
      errorUpdateCount: 0,
      failureCount: 0,
      hasNextPage: hasNextPage(options, data),
      hasPreviousPage: hasPreviousPage(options, data),
      isFetching: false,
      isFetchingNextPage: false,
      isFetchingPreviousPage: false,
      isInvalidated: false,
      pageParams: undefined,
      status: hasData ? 'success' : 'idle',
      updatedAt: hasData ? Date.now() : 0
    };
  }

  function queryReducer(state, action) {
    var _action$updatedAt;

    switch (action.type) {
      case 'failed':
        return _extends({}, state, {
          failureCount: state.failureCount + 1
        });

      case 'fetch':
        return _extends({}, state, {
          failureCount: 0,
          isFetching: true,
          isFetchingNextPage: action.isFetchingNextPage || false,
          isFetchingPreviousPage: action.isFetchingPreviousPage || false,
          status: state.updatedAt ? 'success' : 'loading'
        });

      case 'success':
        return _extends({}, state, {
          data: action.data,
          dataUpdateCount: state.dataUpdateCount + 1,
          error: null,
          failureCount: 0,
          hasNextPage: action.hasNextPage,
          hasPreviousPage: action.hasPreviousPage,
          pageParams: action.pageParams,
          isFetching: false,
          isFetchingNextPage: false,
          isFetchingPreviousPage: false,
          isInvalidated: false,
          status: 'success',
          updatedAt: (_action$updatedAt = action.updatedAt) != null ? _action$updatedAt : Date.now()
        });

      case 'error':
        var error = action.error;

        if (isCancelledError(error) && error.revert) {
          return _extends({}, state, {
            failureCount: 0,
            isFetching: false,
            isFetchingNextPage: false,
            isFetchingPreviousPage: false,
            status: state.error ? 'error' : state.updatedAt ? 'success' : 'idle'
          });
        }

        return _extends({}, state, {
          error: error,
          errorUpdateCount: state.errorUpdateCount + 1,
          failureCount: state.failureCount + 1,
          isFetching: false,
          isFetchingNextPage: false,
          isFetchingPreviousPage: false,
          status: 'error'
        });

      case 'invalidate':
        return _extends({}, state, {
          isInvalidated: true
        });

      default:
        return state;
    }
  }

  // CLASS
  var QueryCache = /*#__PURE__*/function () {
    function QueryCache() {
      this.listeners = [];
      this.queries = [];
      this.queriesMap = {};
    }

    var _proto = QueryCache.prototype;

    _proto.build = function build(options) {
      var _options$queryHash;

      var hashFn = getQueryKeyHashFn(options);
      var queryKey = options.queryKey;
      var queryHash = (_options$queryHash = options.queryHash) != null ? _options$queryHash : hashFn(queryKey);
      var query = this.get(queryHash);

      if (!query) {
        query = new Query({
          cache: this,
          queryKey: queryKey,
          queryHash: queryHash,
          options: options
        });
        this.add(query);
      }

      return query;
    };

    _proto.add = function add(query) {
      if (!this.queriesMap[query.queryHash]) {
        this.queriesMap[query.queryHash] = query;
        this.queries.push(query);
        this.notify(query);
      }
    };

    _proto.remove = function remove(query) {
      if (this.queriesMap[query.queryHash]) {
        query.destroy();
        delete this.queriesMap[query.queryHash];
        this.queries = this.queries.filter(function (x) {
          return x !== query;
        });
        this.notify(query);
      }
    };

    _proto.clear = function clear() {
      var _this = this;

      notifyManager.batch(function () {
        _this.queries.forEach(function (query) {
          _this.remove(query);
        });
      });
    };

    _proto.get = function get(queryHash) {
      return this.queriesMap[queryHash];
    };

    _proto.getAll = function getAll() {
      return this.queries;
    };

    _proto.find = function find(arg1, arg2) {
      var _parseFilterArgs = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs[0];

      return this.queries.find(function (query) {
        return matchQuery(filters, query);
      });
    };

    _proto.findAll = function findAll(arg1, arg2) {
      var _parseFilterArgs2 = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs2[0];

      return filters && Object.keys(filters).length > 0 ? this.queries.filter(function (query) {
        return matchQuery(filters, query);
      }) : this.queries;
    };

    _proto.subscribe = function subscribe(listener) {
      var _this2 = this;

      this.listeners.push(listener);
      return function () {
        _this2.listeners = _this2.listeners.filter(function (x) {
          return x !== listener;
        });
      };
    };

    _proto.notify = function notify(query) {
      var _this3 = this;

      notifyManager.batch(function () {
        _this3.listeners.forEach(function (listener) {
          notifyManager.schedule(function () {
            listener(query);
          });
        });
      });
    };

    return QueryCache;
  }();

  var QueriesObserver = /*#__PURE__*/function () {
    function QueriesObserver(config) {
      this.client = config.client;
      this.queries = config.queries || [];
      this.result = [];
      this.observers = [];
      this.listeners = []; // Subscribe to queries

      this.updateObservers();
    }

    var _proto = QueriesObserver.prototype;

    _proto.subscribe = function subscribe(listener) {
      var _this = this;

      var callback = listener || function () {
        return undefined;
      };

      this.listeners.push(callback);

      if (this.listeners.length === 1) {
        this.onMount();
      }

      return function () {
        _this.unsubscribe(callback);
      };
    };

    _proto.unsubscribe = function unsubscribe(listener) {
      this.listeners = this.listeners.filter(function (x) {
        return x !== listener;
      });

      if (!this.listeners.length) {
        this.clear();
      }
    };

    _proto.onMount = function onMount() {
      var _this2 = this;

      this.observers.forEach(function (observer) {
        observer.subscribe(function (result) {
          _this2.onUpdate(observer, result);
        });
      });
    };

    _proto.clear = function clear() {
      this.listeners = [];
      this.observers.forEach(function (observer) {
        observer.clear();
      });
    };

    _proto.setQueries = function setQueries(queries) {
      this.queries = queries;
      this.updateObservers();
    };

    _proto.getCurrentResult = function getCurrentResult() {
      return this.result;
    };

    _proto.updateObservers = function updateObservers() {
      var _this3 = this;

      var hasIndexChange = false;
      var prevObservers = this.observers;
      var newObservers = this.queries.map(function (options, i) {
        var observer = prevObservers[i];

        var defaultedOptions = _this3.client.defaultQueryObserverOptions(options);

        var hashFn = getQueryKeyHashFn(defaultedOptions);
        defaultedOptions.queryHash = hashFn(defaultedOptions.queryKey);

        if (!observer || observer.getCurrentQuery().queryHash !== defaultedOptions.queryHash) {
          hasIndexChange = true;
          observer = prevObservers.find(function (x) {
            return x.getCurrentQuery().queryHash === defaultedOptions.queryHash;
          });
        }

        if (observer) {
          observer.setOptions(defaultedOptions);
          return observer;
        }

        return _this3.client.watchQuery(defaultedOptions);
      });

      if (prevObservers.length === newObservers.length && !hasIndexChange) {
        return;
      }

      this.observers = newObservers;
      this.result = newObservers.map(function (observer) {
        return observer.getCurrentResult();
      });

      if (!this.listeners.length) {
        return;
      }

      difference(prevObservers, newObservers).forEach(function (observer) {
        observer.clear();
      });
      difference(newObservers, prevObservers).forEach(function (observer) {
        observer.subscribe(function (result) {
          _this3.onUpdate(observer, result);
        });
      });
      this.notify();
    };

    _proto.onUpdate = function onUpdate(observer, result) {
      var index = this.observers.indexOf(observer);

      if (index !== -1) {
        this.result = replaceAt(this.result, index, result);
        this.notify();
      }
    };

    _proto.notify = function notify() {
      var result = this.result,
          listeners = this.listeners;
      notifyManager.batch(function () {
        listeners.forEach(function (listener) {
          notifyManager.schedule(function () {
            listener(result);
          });
        });
      });
    };

    return QueriesObserver;
  }();

  var QueryObserver = /*#__PURE__*/function () {
    function QueryObserver(config) {
      this.client = config.client;
      this.options = config.client.defaultQueryObserverOptions(config.options);
      this.listeners = [];
      this.initialDataUpdateCount = 0; // Bind exposed methods

      this.remove = this.remove.bind(this);
      this.refetch = this.refetch.bind(this);
      this.fetchNextPage = this.fetchNextPage.bind(this);
      this.fetchPreviousPage = this.fetchPreviousPage.bind(this); // Subscribe to the query

      this.updateQuery();
    }

    var _proto = QueryObserver.prototype;

    _proto.subscribe = function subscribe(listener) {
      var _this = this;

      var callback = listener || function () {
        return undefined;
      };

      this.listeners.push(callback);

      if (this.listeners.length === 1) {
        this.onMount();
      }

      return function () {
        _this.unsubscribe(callback);
      };
    };

    _proto.unsubscribe = function unsubscribe(listener) {
      this.listeners = this.listeners.filter(function (x) {
        return x !== listener;
      });

      if (!this.listeners.length) {
        this.clear();
      }
    };

    _proto.onMount = function onMount() {
      this.currentQuery.subscribeObserver(this);

      if (this.willFetchOnMount()) {
        this.executeFetch();
      }

      this.updateTimers();
    };

    _proto.willFetchOnMount = function willFetchOnMount() {
      return this.options.enabled !== false && (!this.currentQuery.state.updatedAt || this.options.refetchOnMount === 'always' || this.options.refetchOnMount !== false && this.isStale());
    };

    _proto.willFetchOptionally = function willFetchOptionally() {
      return this.options.enabled !== false && this.isStale();
    };

    _proto.isStale = function isStale() {
      return this.currentQuery.isStaleByTime(this.options.staleTime);
    };

    _proto.clear = function clear() {
      this.listeners = [];
      this.clearTimers();
      this.currentQuery.unsubscribeObserver(this);
    };

    _proto.setOptions = function setOptions(options) {
      var prevOptions = this.options;
      var prevQuery = this.currentQuery;
      this.options = this.client.defaultQueryObserverOptions(options); // Keep previous query key if the user does not supply one

      if (!this.options.queryKey) {
        this.options.queryKey = prevOptions.queryKey;
      }

      this.updateQuery(); // Take no further actions if there are no subscribers

      if (!this.listeners.length) {
        return;
      } // If we subscribed to a new query, optionally fetch and update refetch


      if (this.currentQuery !== prevQuery) {
        this.optionalFetch();
        this.updateTimers();
        return;
      } // Optionally fetch if the query became enabled


      if (this.options.enabled !== false && prevOptions.enabled === false) {
        this.optionalFetch();
      } // Update stale interval if needed


      if (this.options.enabled !== prevOptions.enabled || this.options.staleTime !== prevOptions.staleTime) {
        this.updateStaleTimeout();
      } // Update refetch interval if needed


      if (this.options.enabled !== prevOptions.enabled || this.options.refetchInterval !== prevOptions.refetchInterval) {
        this.updateRefetchInterval();
      }
    };

    _proto.getCurrentResult = function getCurrentResult() {
      return this.currentResult;
    };

    _proto.getCurrentOrNextResult = function getCurrentOrNextResult(options) {
      if (this.currentQuery.isFetching()) {
        return this.getNextResult(options);
      } else if (this.currentResult.isError && (options == null ? void 0 : options.throwOnError)) {
        return Promise.reject(this.currentResult.error);
      }

      return Promise.resolve(this.currentResult);
    };

    _proto.getNextResult = function getNextResult(options) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var unsubscribe = _this2.subscribe(function (result) {
          if (!result.isFetching) {
            unsubscribe();

            if (result.isError && (options == null ? void 0 : options.throwOnError)) {
              reject(result.error);
            } else {
              resolve(result);
            }
          }
        });
      });
    };

    _proto.getCurrentQuery = function getCurrentQuery() {
      return this.currentQuery;
    };

    _proto.remove = function remove() {
      this.currentQuery.remove();
    };

    _proto.refetch = function refetch(options) {
      return this.fetch(options);
    };

    _proto.fetchNextPage = function fetchNextPage(options) {
      return this.fetch({
        throwOnError: options == null ? void 0 : options.throwOnError,
        fetchMore: {
          direction: 'forward',
          pageParam: options == null ? void 0 : options.pageParam
        }
      });
    };

    _proto.fetchPreviousPage = function fetchPreviousPage(options) {
      return this.fetch({
        throwOnError: options == null ? void 0 : options.throwOnError,
        fetchMore: {
          direction: 'backward',
          pageParam: options == null ? void 0 : options.pageParam
        }
      });
    };

    _proto.fetch = function fetch(fetchOptions) {
      if (!this.canFetch()) {
        return this.getCurrentOrNextResult();
      }

      var promise = this.getNextResult(fetchOptions);
      this.executeFetch(fetchOptions);
      return promise;
    };

    _proto.optionalFetch = function optionalFetch() {
      if (this.willFetchOptionally()) {
        this.executeFetch();
      }
    };

    _proto.executeFetch = function executeFetch(fetchOptions) {
      if (this.canFetch()) {
        this.currentQuery.fetch(this.getQueryOptions(), fetchOptions).catch(noop);
      }
    };

    _proto.canFetch = function canFetch() {
      var _this$currentQuery$de;

      return Boolean(this.options.queryFn || ((_this$currentQuery$de = this.currentQuery.defaultOptions) == null ? void 0 : _this$currentQuery$de.queryFn));
    };

    _proto.updateStaleTimeout = function updateStaleTimeout() {
      var _this3 = this;

      this.clearStaleTimeout();

      if (isServer || this.currentResult.isStale || !isValidTimeout(this.options.staleTime)) {
        return;
      }

      var time = timeUntilStale(this.currentResult.updatedAt, this.options.staleTime); // The timeout is sometimes triggered 1 ms before the stale time expiration.
      // To mitigate this issue we always add 1 ms to the timeout.

      var timeout = time + 1;
      this.staleTimeoutId = setTimeout(function () {
        if (!_this3.currentResult.isStale) {
          _this3.updateResult();

          _this3.notify({
            listeners: true,
            cache: true
          });
        }
      }, timeout);
    };

    _proto.updateRefetchInterval = function updateRefetchInterval() {
      var _this4 = this;

      this.clearRefetchInterval();

      if (isServer || this.options.enabled === false || !isValidTimeout(this.options.refetchInterval)) {
        return;
      }

      this.refetchIntervalId = setInterval(function () {
        if (_this4.options.refetchIntervalInBackground || isDocumentVisible()) {
          _this4.executeFetch();
        }
      }, this.options.refetchInterval);
    };

    _proto.updateTimers = function updateTimers() {
      this.updateStaleTimeout();
      this.updateRefetchInterval();
    };

    _proto.clearTimers = function clearTimers() {
      this.clearStaleTimeout();
      this.clearRefetchInterval();
    };

    _proto.clearStaleTimeout = function clearStaleTimeout() {
      clearInterval(this.staleTimeoutId);
      this.staleTimeoutId = undefined;
    };

    _proto.clearRefetchInterval = function clearRefetchInterval() {
      clearInterval(this.refetchIntervalId);
      this.refetchIntervalId = undefined;
    };

    _proto.getQueryOptions = function getQueryOptions() {
      return this.options;
    };

    _proto.updateResult = function updateResult(willFetch) {
      var _this$previousQueryRe;

      var state = this.currentQuery.state;
      var status = state.status,
          isFetching = state.isFetching,
          updatedAt = state.updatedAt;
      var isPreviousData = false;
      var data; // Already set the status to loading if we are going to fetch

      if (willFetch) {
        isFetching = true;

        if (status === 'idle') {
          status = 'loading';
        }
      } // Keep previous data if needed


      if (this.options.keepPreviousData && !state.dataUpdateCount && ((_this$previousQueryRe = this.previousQueryResult) == null ? void 0 : _this$previousQueryRe.isSuccess)) {
        data = this.previousQueryResult.data;
        updatedAt = this.previousQueryResult.updatedAt;
        status = this.previousQueryResult.status;
        isPreviousData = true;
      } else if (this.options.select && typeof state.data !== 'undefined') {
        var _this$currentResultSt;

        // Use the previous select result if the query data did not change
        if (this.currentResult && state.data === ((_this$currentResultSt = this.currentResultState) == null ? void 0 : _this$currentResultSt.data)) {
          data = this.currentResult.data;
        } else {
          data = this.options.select(state.data);

          if (this.options.structuralSharing !== false) {
            var _this$currentResult;

            data = replaceEqualDeep((_this$currentResult = this.currentResult) == null ? void 0 : _this$currentResult.data, data);
          }
        }
      } else {
        data = state.data;
      }

      var result = _extends({}, getStatusProps(status), {
        data: data,
        error: state.error,
        failureCount: state.failureCount,
        fetchNextPage: this.fetchNextPage,
        fetchPreviousPage: this.fetchPreviousPage,
        hasNextPage: state.hasNextPage,
        hasPreviousPage: state.hasPreviousPage,
        isFetched: state.dataUpdateCount > 0,
        isFetchedAfterMount: state.dataUpdateCount > this.initialDataUpdateCount,
        isFetching: isFetching,
        isFetchingNextPage: state.isFetchingNextPage,
        isFetchingPreviousPage: state.isFetchingPreviousPage,
        isPreviousData: isPreviousData,
        isStale: this.isStale(),
        refetch: this.refetch,
        remove: this.remove,
        updatedAt: updatedAt
      }); // Keep reference to the current state on which the current result is based on


      this.currentResultState = state; // Only update if something has changed

      if (!this.currentResult || !shallowEqualObjects(this.currentResult, result)) {
        this.currentResult = result;
      }
    };

    _proto.updateQuery = function updateQuery() {
      var prevQuery = this.currentQuery;
      var queryOptions = this.getQueryOptions();
      var query = this.client.getCache().build(queryOptions);

      if (query === prevQuery) {
        return;
      }

      this.previousQueryResult = this.currentResult;
      this.currentQuery = query;
      this.initialDataUpdateCount = query.state.dataUpdateCount;
      var willFetch = prevQuery ? this.willFetchOptionally() : this.willFetchOnMount();
      this.updateResult(willFetch);

      if (!this.listeners.length) {
        return;
      }

      prevQuery == null ? void 0 : prevQuery.unsubscribeObserver(this);
      this.currentQuery.subscribeObserver(this);

      if (this.options.notifyOnStatusChange !== false) {
        this.notify({
          listeners: true
        });
      }
    };

    _proto.onQueryUpdate = function onQueryUpdate(action) {
      var options = this.options;
      var type = action.type; // Store current result and get new result

      var prevResult = this.currentResult;
      this.updateResult();
      var currentResult = this.currentResult; // Update timers if needed

      if (type === 'success' || type === 'error' || type === 'invalidate') {
        this.updateTimers();
      } // Do not notify if the nothing has changed


      if (prevResult === currentResult) {
        return;
      } // Determine which callbacks to trigger


      var notifyOptions = {};

      if (type === 'success') {
        notifyOptions.onSuccess = true;
      } else if (type === 'error') {
        notifyOptions.onError = true;
      }

      if ( // Always notify if notifyOnStatusChange is set
      options.notifyOnStatusChange !== false || // Otherwise only notify on data or error change
      currentResult.data !== prevResult.data || currentResult.error !== prevResult.error) {
        notifyOptions.listeners = true;
      }

      this.notify(notifyOptions);
    };

    _proto.notify = function notify(notifyOptions) {
      var _this5 = this;

      var options = this.options,
          currentResult = this.currentResult,
          currentQuery = this.currentQuery,
          listeners = this.listeners;
      var onSuccess = options.onSuccess,
          onSettled = options.onSettled,
          onError = options.onError;
      notifyManager.batch(function () {
        // First trigger the configuration callbacks
        if (notifyOptions.onSuccess) {
          if (onSuccess) {
            notifyManager.schedule(function () {
              onSuccess(currentResult.data);
            });
          }

          if (onSettled) {
            notifyManager.schedule(function () {
              onSettled(currentResult.data, null);
            });
          }
        } else if (notifyOptions.onError) {
          if (onError) {
            notifyManager.schedule(function () {
              onError(currentResult.error);
            });
          }

          if (onSettled) {
            notifyManager.schedule(function () {
              onSettled(undefined, currentResult.error);
            });
          }
        } // Then trigger the listeners


        if (notifyOptions.listeners) {
          listeners.forEach(function (listener) {
            notifyManager.schedule(function () {
              listener(currentResult);
            });
          });
        } // Then the cache listeners


        if (notifyOptions.cache) {
          _this5.client.getCache().notify(currentQuery);
        }
      });
    };

    return QueryObserver;
  }();

  var focusCallback = noop;
  var removePreviousHandler;
  function setFocusHandler(callback) {
    if (removePreviousHandler) {
      removePreviousHandler();
    }

    removePreviousHandler = callback(function () {
      focusCallback();
    });
  }
  function initFocusHandler(callback) {
    // Set the callback to execute on focus
    focusCallback = callback; // Set a default focus handler if needed

    if (!removePreviousHandler) setFocusHandler(function (handleFocus) {
      var _window;

      if (!isServer && ((_window = window) == null ? void 0 : _window.addEventListener)) {
        // Listen to visibillitychange and focus
        window.addEventListener('visibilitychange', handleFocus, false);
        window.addEventListener('focus', handleFocus, false);
        return function () {
          // Be sure to unsubscribe if a new handler is set
          window.removeEventListener('visibilitychange', handleFocus);
          window.removeEventListener('focus', handleFocus);
        };
      }
    });
  }

  var onlineCallback = noop;
  var removePreviousHandler$1;
  function setOnlineHandler(callback) {
    if (removePreviousHandler$1) {
      removePreviousHandler$1();
    }

    removePreviousHandler$1 = callback(function () {
      onlineCallback();
    });
  }
  function initOnlineHandler(callback) {
    // Set the callback to execute on online
    onlineCallback = callback; // Set a default focus handler if needed

    if (!removePreviousHandler$1) {
      setOnlineHandler(function (handleOnline) {
        var _window;

        if (!isServer && ((_window = window) == null ? void 0 : _window.addEventListener)) {
          // Listen to online
          window.addEventListener('online', handleOnline, false);
          return function () {
            // Be sure to unsubscribe if a new handler is set
            window.removeEventListener('online', handleOnline);
          };
        }
      });
    }
  }

  // CLASS
  var QueryClient = /*#__PURE__*/function () {
    function QueryClient(config) {
      this.cache = config.cache;
      this.defaultOptions = config.defaultOptions || {};
    }

    var _proto = QueryClient.prototype;

    _proto.getDefaultOptions = function getDefaultOptions() {
      return this.defaultOptions;
    };

    _proto.setDefaultOptions = function setDefaultOptions(options) {
      this.defaultOptions = options;
    };

    _proto.mount = function mount() {
      mountedClients.push(this);
      initFocusHandler(onFocus);
      initOnlineHandler(onOnline);
    };

    _proto.unmount = function unmount() {
      var index = mountedClients.indexOf(this);

      if (index > -1) {
        mountedClients.splice(index, 1);
      }
    };

    _proto.isFetching = function isFetching(arg1, arg2) {
      var _parseFilterArgs = parseFilterArgs(arg1, arg2),
          filters = _parseFilterArgs[0];

      filters.fetching = true;
      return this.cache.findAll(filters).length;
    };

    _proto.setQueryDefaults = function setQueryDefaults(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      this.cache.build(defaultedOptions).setDefaultOptions(defaultedOptions);
    };

    _proto.getQueryData = function getQueryData(queryKey, filters) {
      var _this$cache$find;

      return (_this$cache$find = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find.state.data;
    };

    _proto.setQueryData = function setQueryData(queryKey, updater, options) {
      var parsedOptions = parseQueryArgs(queryKey);
      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      return this.cache.build(defaultedOptions).setData(updater, options);
    };

    _proto.getQueryState = function getQueryState(queryKey, filters) {
      var _this$cache$find2;

      return (_this$cache$find2 = this.cache.find(queryKey, filters)) == null ? void 0 : _this$cache$find2.state;
    };

    _proto.removeQueries = function removeQueries(arg1, arg2) {
      var _this = this;

      notifyManager.batch(function () {
        _this.cache.findAll(arg1, arg2).forEach(function (query) {
          _this.cache.remove(query);
        });
      });
    };

    _proto.cancelQueries = function cancelQueries(arg1, arg2, arg3) {
      var _this2 = this;

      var _parseFilterArgs2 = parseFilterArgs(arg1, arg2, arg3),
          filters = _parseFilterArgs2[0],
          options = _parseFilterArgs2[1];

      var cancelOptions = options || {};

      if (typeof cancelOptions.revert === 'undefined') {
        cancelOptions.revert = true;
      }

      var promises = notifyManager.batch(function () {
        return _this2.cache.findAll(filters).map(function (query) {
          return query.cancel(cancelOptions);
        });
      });
      return Promise.all(promises).then(noop).catch(noop);
    };

    _proto.invalidateQueries = function invalidateQueries(arg1, arg2, arg3) {
      var _filters$refetchActiv,
          _filters$refetchInact,
          _this3 = this;

      var _parseFilterArgs3 = parseFilterArgs(arg1, arg2, arg3),
          filters = _parseFilterArgs3[0],
          options = _parseFilterArgs3[1];

      var refetchFilters = _extends({}, filters, {
        active: (_filters$refetchActiv = filters.refetchActive) != null ? _filters$refetchActiv : true,
        inactive: (_filters$refetchInact = filters.refetchInactive) != null ? _filters$refetchInact : false
      });

      return notifyManager.batch(function () {
        _this3.cache.findAll(filters).forEach(function (query) {
          query.invalidate();
        });

        return _this3.refetchQueries(refetchFilters, options);
      });
    };

    _proto.refetchQueries = function refetchQueries(arg1, arg2, arg3) {
      var _this4 = this;

      var _parseFilterArgs4 = parseFilterArgs(arg1, arg2, arg3),
          filters = _parseFilterArgs4[0],
          options = _parseFilterArgs4[1];

      var promises = notifyManager.batch(function () {
        return _this4.cache.findAll(filters).map(function (query) {
          return query.fetch();
        });
      });
      var promise = Promise.all(promises).then(noop);

      if (!(options == null ? void 0 : options.throwOnError)) {
        promise = promise.catch(noop);
      }

      return promise;
    };

    _proto.watchQuery = function watchQuery(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
      return new QueryObserver({
        client: this,
        options: parsedOptions
      });
    };

    _proto.watchQueries = function watchQueries(queries) {
      return new QueriesObserver({
        client: this,
        queries: queries
      });
    };

    _proto.fetchQueryData = function fetchQueryData(arg1, arg2, arg3) {
      var parsedOptions = parseQueryArgs(arg1, arg2, arg3); // https://github.com/tannerlinsley/react-query/issues/652

      if (typeof parsedOptions.retry === 'undefined') {
        parsedOptions.retry = false;
      }

      var defaultedOptions = this.defaultQueryOptions(parsedOptions);
      var query = this.cache.find(defaultedOptions.queryKey);

      if (!query) {
        query = this.cache.build(defaultedOptions);
      } else if (!query.isStaleByTime(defaultedOptions.staleTime)) {
        return Promise.resolve(query.state.data);
      }

      return query.fetch(defaultedOptions);
    };

    _proto.prefetchQuery = function prefetchQuery(arg1, arg2, arg3) {
      return this.fetchQueryData(arg1, arg2, arg3).then(noop).catch(noop);
    };

    _proto.getCache = function getCache() {
      return this.cache;
    };

    _proto.defaultQueryOptions = function defaultQueryOptions(options) {
      return _extends({}, this.defaultOptions.queries, options);
    };

    _proto.defaultQueryObserverOptions = function defaultQueryObserverOptions(options) {
      return _extends({}, this.defaultOptions.queries, options);
    };

    _proto.defaultMutationOptions = function defaultMutationOptions(options) {
      return _extends({}, this.defaultOptions.mutations, options);
    };

    return QueryClient;
  }();
  var mountedClients = [];

  function onFocus() {
    onExternalEvent('focus');
  }

  function onOnline() {
    onExternalEvent('online');
  }

  function onExternalEvent(type) {
    if (isDocumentVisible() && isOnline()) {
      notifyManager.batch(function () {
        uniq(mountedClients.map(function (x) {
          return x.getCache();
        })).forEach(function (cache) {
          cache.getAll().forEach(function (query) {
            if (type === 'focus') {
              query.onFocus();
            } else {
              query.onOnline();
            }
          });
        });
      });
    }
  }

  var QueryClientContext = /*#__PURE__*/React.createContext(undefined);
  var useQueryClient = function useQueryClient() {
    return React.useContext(QueryClientContext);
  };
  var QueryClientProvider = function QueryClientProvider(_ref) {
    var client = _ref.client,
        children = _ref.children;
    React.useEffect(function () {
      client.mount();
      return function () {
        client.unmount();
      };
    }, [client]);
    return /*#__PURE__*/React.createElement(QueryClientContext.Provider, {
      value: client
    }, children);
  };

  function createValue() {
    var _isReset = false;
    return {
      clearReset: function clearReset() {
        _isReset = false;
      },
      reset: function reset() {
        _isReset = true;
      },
      isReset: function isReset() {
        return _isReset;
      }
    };
  }

  var QueryErrorResetBoundaryContext = /*#__PURE__*/React.createContext(createValue()); // HOOK

  var useQueryErrorResetBoundary = function useQueryErrorResetBoundary() {
    return React.useContext(QueryErrorResetBoundaryContext);
  }; // COMPONENT

  var QueryErrorResetBoundary = function QueryErrorResetBoundary(_ref) {
    var children = _ref.children;
    var value = React.useMemo(function () {
      return createValue();
    }, []);
    return /*#__PURE__*/React.createElement(QueryErrorResetBoundaryContext.Provider, {
      value: value
    }, typeof children === 'function' ? children(value) : children);
  };

  function useIsMounted() {
    var mountedRef = React.useRef(false);
    var isMounted = React.useCallback(function () {
      return mountedRef.current;
    }, []);
    var useEffect = isServer ? React.useEffect : React.useLayoutEffect;
    useEffect(function () {
      mountedRef.current = true;
      return function () {
        mountedRef.current = false;
      };
    }, []);
    return isMounted;
  }

  function useIsFetching(arg1, arg2) {
    var client = useQueryClient();
    var isMounted = useIsMounted();

    var _parseFilterArgs = parseFilterArgs(arg1, arg2),
        filters = _parseFilterArgs[0];

    var _React$useState = React.useState(client.isFetching(filters)),
        isFetching = _React$useState[0],
        setIsFetching = _React$useState[1];

    var filtersRef = React.useRef(filters);
    filtersRef.current = filters;
    var isFetchingRef = React.useRef(isFetching);
    isFetchingRef.current = isFetching;
    React.useEffect(function () {
      return client.getCache().subscribe(function () {
        if (isMounted()) {
          var newIsFetching = client.isFetching(filtersRef.current);

          if (isFetchingRef.current !== newIsFetching) {
            setIsFetching(newIsFetching);
          }
        }
      });
    }, [client, setIsFetching, isMounted]);
    return isFetching;
  }

  // HOOK
  function getDefaultState$1() {
    return _extends({}, getStatusProps('idle'), {
      data: undefined,
      error: null
    });
  }

  function reducer(state, action) {
    switch (action.type) {
      case 'reset':
        return getDefaultState$1();

      case 'loading':
        return _extends({}, getStatusProps('loading'), {
          data: undefined,
          error: null
        });

      case 'success':
        return _extends({}, getStatusProps('success'), {
          data: action.data,
          error: null
        });

      case 'error':
        return _extends({}, getStatusProps('error'), {
          data: undefined,
          error: action.error
        });

      default:
        return state;
    }
  }

  function useMutation(mutationFn, options) {
    if (options === void 0) {
      options = {};
    }

    var isMounted = useIsMounted();

    var _React$useReducer = React.useReducer(reducer, null, getDefaultState$1),
        state = _React$useReducer[0],
        dispatch = _React$useReducer[1];

    var safeDispatch = React.useCallback(function (action) {
      notifyManager.schedule(function () {
        if (isMounted()) {
          dispatch(action);
        }
      });
    }, [dispatch, isMounted]);
    var client = useQueryClient();
    var defaultedOptions = client.defaultMutationOptions(options);
    var lastMutationIdRef = React.useRef(0);
    var lastMutationFnRef = React.useRef(mutationFn);
    lastMutationFnRef.current = mutationFn;
    var lastOptionsRef = React.useRef(defaultedOptions);
    lastOptionsRef.current = defaultedOptions;
    var mutateAsync = React.useCallback(function (vars, mutateOpts) {
      if (mutateOpts === void 0) {
        mutateOpts = {};
      }

      var mutationId = ++lastMutationIdRef.current;
      var mutationOpts = lastOptionsRef.current;
      var lastMutationFn = lastMutationFnRef.current;
      var ctx;
      var data;
      safeDispatch({
        type: 'loading'
      });
      return Promise.resolve().then(function () {
        return mutationOpts.onMutate == null ? void 0 : mutationOpts.onMutate(vars);
      }).then(function (context) {
        ctx = context;
      }).then(function () {
        return lastMutationFn(vars);
      }).then(function (result) {
        data = result;
      }).then(function () {
        return mutationOpts.onSuccess == null ? void 0 : mutationOpts.onSuccess(data, vars, ctx);
      }).then(function () {
        return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(data, null, vars, ctx);
      }).then(function () {
        return mutateOpts.onSuccess == null ? void 0 : mutateOpts.onSuccess(data, vars, ctx);
      }).then(function () {
        return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(data, null, vars, ctx);
      }).then(function () {
        if (lastMutationIdRef.current === mutationId) {
          safeDispatch({
            type: 'success',
            data: data
          });
        }

        return data;
      }).catch(function (error) {
        getLogger().error(error);
        return Promise.resolve().then(function () {
          return mutationOpts.onError == null ? void 0 : mutationOpts.onError(error, vars, ctx);
        }).then(function () {
          return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(undefined, error, vars, ctx);
        }).then(function () {
          return mutateOpts.onError == null ? void 0 : mutateOpts.onError(error, vars, ctx);
        }).then(function () {
          return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(undefined, error, vars, ctx);
        }).then(function () {
          if (lastMutationIdRef.current === mutationId) {
            safeDispatch({
              type: 'error',
              error: error
            });
          }

          throw error;
        });
      });
    }, [safeDispatch]);
    var mutate = React.useCallback(function (variables, mutateOptions) {
      mutateAsync(variables, mutateOptions).catch(noop);
    }, [mutateAsync]);
    var reset = React.useCallback(function () {
      safeDispatch({
        type: 'reset'
      });
    }, [safeDispatch]);
    React.useEffect(function () {
      var lastOptions = lastOptionsRef.current;

      if (state.error && (lastOptions.useErrorBoundary || lastOptions.suspense)) {
        throw state.error;
      }
    }, [state.error]);
    return _extends({}, state, {
      mutate: mutate,
      mutateAsync: mutateAsync,
      reset: reset
    });
  }

  function useBaseQuery(options) {
    var client = useQueryClient();
    var isMounted = useIsMounted();
    var errorResetBoundary = useQueryErrorResetBoundary();
    var defaultedOptions = client.defaultQueryObserverOptions(options); // Always set stale time when using suspense

    if (defaultedOptions.suspense && !defaultedOptions.staleTime) {
      defaultedOptions.staleTime = 5000;
    } // Create query observer


    var observerRef = React.useRef();
    var firstRender = !observerRef.current;
    var observer = observerRef.current || client.watchQuery(defaultedOptions);
    observerRef.current = observer; // Update options

    if (!firstRender) {
      observer.setOptions(defaultedOptions);
    }

    var _React$useState = React.useState(function () {
      return observer.getCurrentResult();
    }),
        currentResult = _React$useState[0],
        setCurrentResult = _React$useState[1]; // Subscribe to the observer


    React.useEffect(function () {
      errorResetBoundary.clearReset();
      return observer.subscribe(function (result) {
        if (isMounted()) {
          setCurrentResult(result);
        }
      });
    }, [isMounted, observer, errorResetBoundary]); // Handle suspense

    if (observer.options.suspense || observer.options.useErrorBoundary) {
      if (currentResult.isError && !errorResetBoundary.isReset()) {
        throw currentResult.error;
      }

      if (observer.options.suspense && firstRender && observer.willFetchOnMount()) {
        errorResetBoundary.clearReset();
        throw observer.getNextResult({
          throwOnError: true
        });
      }
    }

    return currentResult;
  }

  function useQuery(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    return useBaseQuery(parsedOptions);
  }

  function useQueries(queries) {
    var client = useQueryClient();
    var isMounted = useIsMounted(); // Create queries observer

    var observerRef = React.useRef();
    var firstRender = !observerRef.current;
    var observer = observerRef.current || client.watchQueries(queries);
    observerRef.current = observer; // Update queries

    if (!firstRender) {
      observer.setQueries(queries);
    }

    var _React$useState = React.useState(function () {
      return observer.getCurrentResult();
    }),
        currentResult = _React$useState[0],
        setCurrentResult = _React$useState[1]; // Subscribe to the observer


    React.useEffect(function () {
      return observer.subscribe(function (result) {
        if (isMounted()) {
          setCurrentResult(result);
        }
      });
    }, [isMounted, observer]);
    return currentResult;
  }

  function useInfiniteQuery(arg1, arg2, arg3) {
    var parsedOptions = parseQueryArgs(arg1, arg2, arg3);
    parsedOptions.infinite = true;
    return useBaseQuery(parsedOptions);
  }

  exports.QueryCache = QueryCache;
  exports.QueryClient = QueryClient;
  exports.QueryClientProvider = QueryClientProvider;
  exports.QueryErrorResetBoundary = QueryErrorResetBoundary;
  exports.hashQueryKey = hashQueryKey;
  exports.isCancelledError = isCancelledError;
  exports.isError = isError;
  exports.setBatchNotifyFn = setBatchNotifyFn;
  exports.setFocusHandler = setFocusHandler;
  exports.setLogger = setLogger;
  exports.setNotifyFn = setNotifyFn;
  exports.setOnlineHandler = setOnlineHandler;
  exports.useInfiniteQuery = useInfiniteQuery;
  exports.useIsFetching = useIsFetching;
  exports.useMutation = useMutation;
  exports.useQueries = useQueries;
  exports.useQuery = useQuery;
  exports.useQueryClient = useQueryClient;
  exports.useQueryErrorResetBoundary = useQueryErrorResetBoundary;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=react-query.development.js.map
