"use strict";

exports.__esModule = true;
exports.setFocusHandler = setFocusHandler;
exports.initFocusHandler = initFocusHandler;

var _utils = require("./utils");

var focusCallback = _utils.noop;
var removePreviousHandler;

function setFocusHandler(callback) {
  if (removePreviousHandler) {
    removePreviousHandler();
  }

  removePreviousHandler = callback(function () {
    focusCallback();
  });
}

function initFocusHandler(callback) {
  // Set the callback to execute on focus
  focusCallback = callback; // Set a default focus handler if needed

  if (!removePreviousHandler) setFocusHandler(function (handleFocus) {
    var _window;

    if (!_utils.isServer && ((_window = window) == null ? void 0 : _window.addEventListener)) {
      // Listen to visibillitychange and focus
      window.addEventListener('visibilitychange', handleFocus, false);
      window.addEventListener('focus', handleFocus, false);
      return function () {
        // Be sure to unsubscribe if a new handler is set
        window.removeEventListener('visibilitychange', handleFocus);
        window.removeEventListener('focus', handleFocus);
      };
    }
  });
}