"use strict";

exports.__esModule = true;
exports.setNotifyFn = setNotifyFn;
exports.setBatchNotifyFn = setBatchNotifyFn;
exports.notifyManager = void 0;

var _utils = require("./utils");

// GETTERS AND SETTERS
// Default to a dummy "notify" implementation that just runs the callback
var notifyFn = function notifyFn(callback) {
  callback();
}; // Default to a dummy "batch notify" implementation that just runs the callback


var batchNotifyFn = function batchNotifyFn(callback) {
  callback();
};
/**
 * Use this function to set a custom notify function.
 * This can be used to for example wrap notifications with `React.act` while running tests.
 */


function setNotifyFn(fn) {
  notifyFn = fn;
}
/**
 * Use this function to set a custom function to batch notifications together into a single tick.
 * By default React Query will use the batch function provided by ReactDOM or React Native.
 */


function setBatchNotifyFn(fn) {
  batchNotifyFn = fn;
} // CLASS


var NotifyManager = /*#__PURE__*/function () {
  function NotifyManager() {
    this.queue = [];
    this.transactions = 0;
  }

  var _proto = NotifyManager.prototype;

  _proto.batch = function batch(callback) {
    this.transactions++;
    var result = callback();
    this.transactions--;

    if (!this.transactions) {
      this.flush();
    }

    return result;
  };

  _proto.schedule = function schedule(callback) {
    if (this.transactions) {
      this.queue.push(callback);
    } else {
      (0, _utils.scheduleMicrotask)(function () {
        notifyFn(callback);
      });
    }
  };

  _proto.flush = function flush() {
    var queue = this.queue;
    this.queue = [];

    if (queue.length) {
      (0, _utils.scheduleMicrotask)(function () {
        batchNotifyFn(function () {
          queue.forEach(function (callback) {
            notifyFn(callback);
          });
        });
      });
    }
  };

  return NotifyManager;
}(); // SINGLETON


var notifyManager = new NotifyManager();
exports.notifyManager = notifyManager;