"use strict";

exports.__esModule = true;
exports.QueryCache = void 0;

var _utils = require("./utils");

var _query = require("./query");

var _notifyManager = require("./notifyManager");

// CLASS
var QueryCache = /*#__PURE__*/function () {
  function QueryCache() {
    this.listeners = [];
    this.queries = [];
    this.queriesMap = {};
  }

  var _proto = QueryCache.prototype;

  _proto.build = function build(options) {
    var _options$queryHash;

    var hashFn = (0, _utils.getQueryKeyHashFn)(options);
    var queryKey = options.queryKey;
    var queryHash = (_options$queryHash = options.queryHash) != null ? _options$queryHash : hashFn(queryKey);
    var query = this.get(queryHash);

    if (!query) {
      query = new _query.Query({
        cache: this,
        queryKey: queryKey,
        queryHash: queryHash,
        options: options
      });
      this.add(query);
    }

    return query;
  };

  _proto.add = function add(query) {
    if (!this.queriesMap[query.queryHash]) {
      this.queriesMap[query.queryHash] = query;
      this.queries.push(query);
      this.notify(query);
    }
  };

  _proto.remove = function remove(query) {
    if (this.queriesMap[query.queryHash]) {
      query.destroy();
      delete this.queriesMap[query.queryHash];
      this.queries = this.queries.filter(function (x) {
        return x !== query;
      });
      this.notify(query);
    }
  };

  _proto.clear = function clear() {
    var _this = this;

    _notifyManager.notifyManager.batch(function () {
      _this.queries.forEach(function (query) {
        _this.remove(query);
      });
    });
  };

  _proto.get = function get(queryHash) {
    return this.queriesMap[queryHash];
  };

  _proto.getAll = function getAll() {
    return this.queries;
  };

  _proto.find = function find(arg1, arg2) {
    var _parseFilterArgs = (0, _utils.parseFilterArgs)(arg1, arg2),
        filters = _parseFilterArgs[0];

    return this.queries.find(function (query) {
      return (0, _utils.matchQuery)(filters, query);
    });
  };

  _proto.findAll = function findAll(arg1, arg2) {
    var _parseFilterArgs2 = (0, _utils.parseFilterArgs)(arg1, arg2),
        filters = _parseFilterArgs2[0];

    return filters && Object.keys(filters).length > 0 ? this.queries.filter(function (query) {
      return (0, _utils.matchQuery)(filters, query);
    }) : this.queries;
  };

  _proto.subscribe = function subscribe(listener) {
    var _this2 = this;

    this.listeners.push(listener);
    return function () {
      _this2.listeners = _this2.listeners.filter(function (x) {
        return x !== listener;
      });
    };
  };

  _proto.notify = function notify(query) {
    var _this3 = this;

    _notifyManager.notifyManager.batch(function () {
      _this3.listeners.forEach(function (listener) {
        _notifyManager.notifyManager.schedule(function () {
          listener(query);
        });
      });
    });
  };

  return QueryCache;
}();

exports.QueryCache = QueryCache;