"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.dehydrate = dehydrate;
exports.hydrate = hydrate;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _query = require("../core/query");

// FUNCTIONS
function serializePositiveNumber(value) {
  return value === Infinity ? -1 : value;
}

function deserializePositiveNumber(value) {
  return value === -1 ? Infinity : value;
} // Most config is not dehydrated but instead meant to configure again when
// consuming the de/rehydrated data, typically with useQuery on the client.
// Sometimes it might make sense to prefetch data on the server and include
// in the html-payload, but not consume it on the initial render.


function dehydrateQuery(query) {
  return {
    config: {
      cacheTime: serializePositiveNumber(query.cacheTime)
    },
    data: query.state.data,
    queryKey: query.queryKey,
    queryHash: query.queryHash,
    updatedAt: query.state.updatedAt
  };
}

function defaultShouldDehydrate(query) {
  return query.state.status === 'success';
}

function dehydrate(cache, options) {
  options = options || {};
  var shouldDehydrate = options.shouldDehydrate || defaultShouldDehydrate;
  var queries = [];
  cache.getAll().forEach(function (query) {
    if (shouldDehydrate(query)) {
      queries.push(dehydrateQuery(query));
    }
  });
  return {
    queries: queries
  };
}

function hydrate(cache, dehydratedState, options) {
  if (typeof dehydratedState !== 'object' || dehydratedState === null) {
    return;
  }

  var defaultOptions = (options == null ? void 0 : options.defaultOptions) || {};
  var queries = dehydratedState.queries || [];
  queries.forEach(function (dehydratedQuery) {
    var query = cache.get(dehydratedQuery.queryHash); // Do not hydrate if an existing query exists with newer data

    if (query && query.state.updatedAt >= dehydratedQuery.updatedAt) {
      return;
    }

    if (!query) {
      query = new _query.Query({
        cache: cache,
        queryKey: dehydratedQuery.queryKey,
        queryHash: dehydratedQuery.queryHash,
        options: (0, _extends2.default)({}, defaultOptions, {
          cacheTime: deserializePositiveNumber(dehydratedQuery.config.cacheTime)
        })
      });
      cache.add(query);
    }

    query.setData(dehydratedQuery.data, {
      updatedAt: dehydratedQuery.updatedAt
    });
  });
}