"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useBaseQuery = useBaseQuery;

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _QueryErrorResetBoundary = require("./QueryErrorResetBoundary");

var _QueryClientProvider = require("./QueryClientProvider");

function useBaseQuery(options) {
  var client = (0, _QueryClientProvider.useQueryClient)();
  var isMounted = (0, _utils.useIsMounted)();
  var errorResetBoundary = (0, _QueryErrorResetBoundary.useQueryErrorResetBoundary)();
  var defaultedOptions = client.defaultQueryObserverOptions(options); // Always set stale time when using suspense

  if (defaultedOptions.suspense && !defaultedOptions.staleTime) {
    defaultedOptions.staleTime = 5000;
  } // Create query observer


  var observerRef = _react.default.useRef();

  var firstRender = !observerRef.current;
  var observer = observerRef.current || client.watchQuery(defaultedOptions);
  observerRef.current = observer; // Update options

  if (!firstRender) {
    observer.setOptions(defaultedOptions);
  }

  var _React$useState = _react.default.useState(function () {
    return observer.getCurrentResult();
  }),
      currentResult = _React$useState[0],
      setCurrentResult = _React$useState[1]; // Subscribe to the observer


  _react.default.useEffect(function () {
    errorResetBoundary.clearReset();
    return observer.subscribe(function (result) {
      if (isMounted()) {
        setCurrentResult(result);
      }
    });
  }, [isMounted, observer, errorResetBoundary]); // Handle suspense


  if (observer.options.suspense || observer.options.useErrorBoundary) {
    if (currentResult.isError && !errorResetBoundary.isReset()) {
      throw currentResult.error;
    }

    if (observer.options.suspense && firstRender && observer.willFetchOnMount()) {
      errorResetBoundary.clearReset();
      throw observer.getNextResult({
        throwOnError: true
      });
    }
  }

  return currentResult;
}