"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useMutation = useMutation;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _utils2 = require("../core/utils");

var _logger = require("../core/logger");

var _notifyManager = require("../core/notifyManager");

var _QueryClientProvider = require("./QueryClientProvider");

// HOOK
function getDefaultState() {
  return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('idle'), {
    data: undefined,
    error: null
  });
}

function reducer(state, action) {
  switch (action.type) {
    case 'reset':
      return getDefaultState();

    case 'loading':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('loading'), {
        data: undefined,
        error: null
      });

    case 'success':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('success'), {
        data: action.data,
        error: null
      });

    case 'error':
      return (0, _extends2.default)({}, (0, _utils2.getStatusProps)('error'), {
        data: undefined,
        error: action.error
      });

    default:
      return state;
  }
}

function useMutation(mutationFn, options) {
  if (options === void 0) {
    options = {};
  }

  var isMounted = (0, _utils.useIsMounted)();

  var _React$useReducer = _react.default.useReducer(reducer, null, getDefaultState),
      state = _React$useReducer[0],
      dispatch = _React$useReducer[1];

  var safeDispatch = _react.default.useCallback(function (action) {
    _notifyManager.notifyManager.schedule(function () {
      if (isMounted()) {
        dispatch(action);
      }
    });
  }, [dispatch, isMounted]);

  var client = (0, _QueryClientProvider.useQueryClient)();
  var defaultedOptions = client.defaultMutationOptions(options);

  var lastMutationIdRef = _react.default.useRef(0);

  var lastMutationFnRef = _react.default.useRef(mutationFn);

  lastMutationFnRef.current = mutationFn;

  var lastOptionsRef = _react.default.useRef(defaultedOptions);

  lastOptionsRef.current = defaultedOptions;

  var mutateAsync = _react.default.useCallback(function (vars, mutateOpts) {
    if (mutateOpts === void 0) {
      mutateOpts = {};
    }

    var mutationId = ++lastMutationIdRef.current;
    var mutationOpts = lastOptionsRef.current;
    var lastMutationFn = lastMutationFnRef.current;
    var ctx;
    var data;
    safeDispatch({
      type: 'loading'
    });
    return Promise.resolve().then(function () {
      return mutationOpts.onMutate == null ? void 0 : mutationOpts.onMutate(vars);
    }).then(function (context) {
      ctx = context;
    }).then(function () {
      return lastMutationFn(vars);
    }).then(function (result) {
      data = result;
    }).then(function () {
      return mutationOpts.onSuccess == null ? void 0 : mutationOpts.onSuccess(data, vars, ctx);
    }).then(function () {
      return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(data, null, vars, ctx);
    }).then(function () {
      return mutateOpts.onSuccess == null ? void 0 : mutateOpts.onSuccess(data, vars, ctx);
    }).then(function () {
      return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(data, null, vars, ctx);
    }).then(function () {
      if (lastMutationIdRef.current === mutationId) {
        safeDispatch({
          type: 'success',
          data: data
        });
      }

      return data;
    }).catch(function (error) {
      (0, _logger.getLogger)().error(error);
      return Promise.resolve().then(function () {
        return mutationOpts.onError == null ? void 0 : mutationOpts.onError(error, vars, ctx);
      }).then(function () {
        return mutationOpts.onSettled == null ? void 0 : mutationOpts.onSettled(undefined, error, vars, ctx);
      }).then(function () {
        return mutateOpts.onError == null ? void 0 : mutateOpts.onError(error, vars, ctx);
      }).then(function () {
        return mutateOpts.onSettled == null ? void 0 : mutateOpts.onSettled(undefined, error, vars, ctx);
      }).then(function () {
        if (lastMutationIdRef.current === mutationId) {
          safeDispatch({
            type: 'error',
            error: error
          });
        }

        throw error;
      });
    });
  }, [safeDispatch]);

  var mutate = _react.default.useCallback(function (variables, mutateOptions) {
    mutateAsync(variables, mutateOptions).catch(_utils2.noop);
  }, [mutateAsync]);

  var reset = _react.default.useCallback(function () {
    safeDispatch({
      type: 'reset'
    });
  }, [safeDispatch]);

  _react.default.useEffect(function () {
    var lastOptions = lastOptionsRef.current;

    if (state.error && (lastOptions.useErrorBoundary || lastOptions.suspense)) {
      throw state.error;
    }
  }, [state.error]);

  return (0, _extends2.default)({}, state, {
    mutate: mutate,
    mutateAsync: mutateAsync,
    reset: reset
  });
}