import { CancelOptions, Updater } from './utils';
import { QueryKey, QueryOptions, QueryStatus } from './types';
import { QueryCache } from './queryCache';
import { QueryObserver } from './queryObserver';
interface QueryConfig<TData, TError, TQueryFnData> {
    cache: QueryCache;
    queryKey: QueryKey;
    queryHash: string;
    options?: QueryOptions<TData, TError, TQueryFnData>;
}
export interface QueryState<TData, TError> {
    data: TData | undefined;
    dataUpdateCount: number;
    error: TError | null;
    errorUpdateCount: number;
    failureCount: number;
    hasNextPage: boolean | undefined;
    hasPreviousPage: boolean | undefined;
    isFetching: boolean;
    isFetchingNextPage: boolean;
    isFetchingPreviousPage: boolean;
    isInvalidated: boolean;
    pageParams: unknown[] | undefined;
    status: QueryStatus;
    updatedAt: number;
}
export interface FetchOptions {
    fetchMore?: FetchMoreOptions;
    throwOnError?: boolean;
}
interface FetchMoreOptions {
    pageParam?: unknown;
    direction: 'forward' | 'backward';
}
export interface SetDataOptions {
    pageParams?: unknown[];
    updatedAt?: number;
}
interface FailedAction {
    type: 'failed';
}
interface FetchAction {
    type: 'fetch';
    isFetchingNextPage?: boolean;
    isFetchingPreviousPage?: boolean;
}
interface SuccessAction<TData> {
    data: TData | undefined;
    hasNextPage?: boolean;
    hasPreviousPage?: boolean;
    pageParams?: unknown[];
    type: 'success';
    updatedAt?: number;
}
interface ErrorAction<TError> {
    type: 'error';
    error: TError;
}
interface InvalidateAction {
    type: 'invalidate';
}
export declare type Action<TData, TError> = ErrorAction<TError> | FailedAction | FetchAction | InvalidateAction | SuccessAction<TData>;
export declare class Query<TData = unknown, TError = unknown, TQueryFnData = TData> {
    queryKey: QueryKey;
    queryHash: string;
    options: QueryOptions<TData, TError, TQueryFnData>;
    defaultOptions?: QueryOptions<TData, TError, TQueryFnData>;
    state: QueryState<TData, TError>;
    cacheTime: number;
    private cache;
    private promise?;
    private gcTimeout?;
    private cancelFetch?;
    private continueFetch?;
    private isTransportCancelable?;
    private observers;
    constructor(config: QueryConfig<TData, TError, TQueryFnData>);
    private setOptions;
    setDefaultOptions(options: QueryOptions<TData, TError, TQueryFnData>): void;
    private dispatch;
    private scheduleGc;
    cancel(options?: CancelOptions): Promise<void>;
    private clearTimersObservers;
    private clearGcTimeout;
    setData(updater: Updater<TData | undefined, TData>, options?: SetDataOptions): TData;
    remove(): void;
    destroy(): void;
    isActive(): boolean;
    isFetching(): boolean;
    isStale(): boolean;
    isStaleByTime(staleTime?: number): boolean;
    onFocus(): void;
    onOnline(): void;
    private onExternalEvent;
    subscribeObserver(observer: QueryObserver<any, any, any, any>): void;
    unsubscribeObserver(observer: QueryObserver<any, any, any, any>): void;
    invalidate(): void;
    fetch(options?: QueryOptions<TData, TError, TQueryFnData>, fetchOptions?: FetchOptions): Promise<TData>;
    private startFetch;
    private startInfiniteFetch;
    private tryFetchData;
}
export {};
