import { CancelOptions, QueryFilters, Updater } from './utils';
import { DefaultOptions, FetchQueryOptions, InvalidateOptions, InvalidateQueryFilters, MutationOptions, QueryFunction, QueryKey, QueryObserverOptions, QueryOptions, RefetchOptions } from './types';
import { QueryState, SetDataOptions } from './query';
import { QueryCache } from './queryCache';
import { QueriesObserver } from './queriesObserver';
import { QueryObserver } from './queryObserver';
interface QueryClientConfig {
    cache: QueryCache;
    defaultOptions?: DefaultOptions;
}
export declare class QueryClient {
    private cache;
    private defaultOptions;
    constructor(config: QueryClientConfig);
    getDefaultOptions(): DefaultOptions;
    setDefaultOptions(options: DefaultOptions): void;
    mount(): void;
    unmount(): void;
    isFetching(filters?: QueryFilters): number;
    isFetching(queryKey?: QueryKey, filters?: QueryFilters): number;
    setQueryDefaults<TData = unknown, TError = unknown, TQueryFnData = TData>(options: QueryOptions<TData, TError, TQueryFnData>): void;
    setQueryDefaults<TData = unknown, TError = unknown, TQueryFnData = TData>(queryKey: QueryKey, options?: QueryOptions<TData, TError, TQueryFnData>): void;
    setQueryDefaults<TData = unknown, TError = unknown, TQueryFnData = TData>(queryKey: QueryKey, queryFn: QueryFunction<TQueryFnData | TData>, options?: QueryOptions<TData, TError, TQueryFnData>): void;
    getQueryData<TData = unknown>(queryKey: QueryKey, filters?: QueryFilters): TData | undefined;
    setQueryData<TData>(queryKey: QueryKey, updater: Updater<TData | undefined, TData>, options?: SetDataOptions): TData;
    getQueryState<TData = unknown, TError = undefined>(queryKey: QueryKey, filters?: QueryFilters): QueryState<TData, TError> | undefined;
    removeQueries(filters?: QueryFilters): void;
    removeQueries(queryKey?: QueryKey, filters?: QueryFilters): void;
    cancelQueries(filters?: QueryFilters, options?: CancelOptions): Promise<void>;
    cancelQueries(queryKey?: QueryKey, filters?: QueryFilters, options?: CancelOptions): Promise<void>;
    invalidateQueries(filters?: InvalidateQueryFilters, options?: InvalidateOptions): Promise<void>;
    invalidateQueries(queryKey?: QueryKey, filters?: InvalidateQueryFilters, options?: InvalidateOptions): Promise<void>;
    refetchQueries(filters?: QueryFilters, options?: RefetchOptions): Promise<void>;
    refetchQueries(queryKey?: QueryKey, filters?: QueryFilters, options?: RefetchOptions): Promise<void>;
    watchQuery<TData = unknown, TError = unknown, TQueryFnData = TData, TQueryData = TQueryFnData>(options: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>): QueryObserver<TData, TError, TQueryFnData, TQueryData>;
    watchQuery<TData = unknown, TError = unknown, TQueryFnData = TData, TQueryData = TQueryFnData>(queryKey: QueryKey, options?: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>): QueryObserver<TData, TError, TQueryFnData, TQueryData>;
    watchQuery<TData = unknown, TError = unknown, TQueryFnData = TData, TQueryData = TQueryFnData>(queryKey: QueryKey, queryFn: QueryFunction<TQueryFnData | TData>, options?: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>): QueryObserver<TData, TError, TQueryFnData, TQueryData>;
    watchQueries(queries: QueryObserverOptions[]): QueriesObserver;
    fetchQueryData<TData = unknown, TError = unknown, TQueryFnData = TData>(options: FetchQueryOptions<TData, TError, TQueryFnData>): Promise<TData>;
    fetchQueryData<TData = unknown, TError = unknown, TQueryFnData = TData>(queryKey: QueryKey, options?: FetchQueryOptions<TData, TError, TQueryFnData>): Promise<TData>;
    fetchQueryData<TData = unknown, TError = unknown, TQueryFnData = TData>(queryKey: QueryKey, queryFn: QueryFunction<TQueryFnData | TData>, options?: FetchQueryOptions<TData, TError, TQueryFnData>): Promise<TData>;
    prefetchQuery(options: FetchQueryOptions): Promise<void>;
    prefetchQuery(queryKey: QueryKey, options?: FetchQueryOptions): Promise<void>;
    prefetchQuery(queryKey: QueryKey, queryFn: QueryFunction, options?: FetchQueryOptions): Promise<void>;
    getCache(): QueryCache;
    defaultQueryOptions<T extends QueryOptions<any, any>>(options?: T): T;
    defaultQueryObserverOptions<T extends QueryObserverOptions<any, any>>(options?: T): T;
    defaultMutationOptions<T extends MutationOptions<any, any, any, any>>(options?: T): T;
}
export {};
