import { FetchNextPageOptions, FetchPreviousPageOptions, QueryObserverOptions, QueryObserverResult, RefetchOptions, ResultOptions } from './types';
import { Query, Action, FetchOptions } from './query';
import { QueryClient } from './queryClient';
interface QueryObserverConfig<TData = unknown, TError = unknown, TQueryFnData = TData, TQueryData = TQueryFnData> {
    client: QueryClient;
    options?: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>;
}
declare type QueryObserverListener<TData, TError> = (result: QueryObserverResult<TData, TError>) => void;
export declare class QueryObserver<TData = unknown, TError = unknown, TQueryFnData = TData, TQueryData = TQueryFnData> {
    options: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>;
    private client;
    private currentQuery;
    private currentResult;
    private currentResultState?;
    private previousQueryResult?;
    private listeners;
    private initialDataUpdateCount;
    private staleTimeoutId?;
    private refetchIntervalId?;
    constructor(config: QueryObserverConfig<TData, TError, TQueryFnData, TQueryData>);
    subscribe(listener?: QueryObserverListener<TData, TError>): () => void;
    private unsubscribe;
    private onMount;
    willFetchOnMount(): boolean;
    willFetchOptionally(): boolean;
    private isStale;
    clear(): void;
    setOptions(options?: QueryObserverOptions<TData, TError, TQueryFnData, TQueryData>): void;
    getCurrentResult(): QueryObserverResult<TData, TError>;
    getCurrentOrNextResult(options?: ResultOptions): Promise<QueryObserverResult<TData, TError>>;
    getNextResult(options?: ResultOptions): Promise<QueryObserverResult<TData, TError>>;
    getCurrentQuery(): Query<TQueryData, TError, TQueryFnData>;
    remove(): void;
    refetch(options?: RefetchOptions): Promise<QueryObserverResult<TData, TError>>;
    fetchNextPage(options?: FetchNextPageOptions): Promise<QueryObserverResult<TData, TError>>;
    fetchPreviousPage(options?: FetchPreviousPageOptions): Promise<QueryObserverResult<TData, TError>>;
    fetch(fetchOptions?: FetchOptions): Promise<QueryObserverResult<TData, TError>>;
    private optionalFetch;
    private executeFetch;
    private canFetch;
    private updateStaleTimeout;
    private updateRefetchInterval;
    updateTimers(): void;
    clearTimers(): void;
    private clearStaleTimeout;
    private clearRefetchInterval;
    private getQueryOptions;
    private updateResult;
    private updateQuery;
    onQueryUpdate(action: Action<TData, TError>): void;
    private notify;
}
export {};
